@@ -153,7 +153,7 @@ pref("keyword.URL", "http://www.google.com/search?btnI=I%27m+Feeling+Lucky&ie=UT
 
 pref("general.useragent.locale", "en-US");
 pref("general.skins.selectedSkin", "classic/1.0");
-pref("general.useragent.extra.firefox", "Firefox/1.5.0.7");
+pref("general.useragent.extra.firefox", "Firefox/1.5.0.8pre");
 
 pref("general.smoothScroll", false);
 //@line 165 "/builds/tinderbox/Fx-Mozilla1.8.0/Linux_2.4.21-37.EL_Depend/mozilla/browser/app/profile/firefox.js"
